# -*- coding: utf-8 -*-

# Coded by Takashi Ida, August 19, 2021

import numpy as np
from scipy import special as sp
from scipy import optimize as opt

#####################
# Truncated Gaussian
#####################
######################################
# Complementary error function erfc(a)
# for the shape parmeter a to kurtosis
#  erfca: erfc(a) (NumPy array)
######################################
def fTG_erfca2k(erfca):
	sqrt_pi = np.sqrt(np.pi)
	erfa = 1 - erfca
	a = sp.erfcinv(erfca)
	x2 = 0.5 - a * np.exp(-a**2)/(sqrt_pi*erfa)
	x4 = 0.25*(3-2*a*(3+2*a**2)*np.exp(-a**2)/(sqrt_pi*erfa))
	kurtosis = x4 / x2**2 - 3
	return kurtosis
#############################
# kurtosis to erfc(a)
#  k0: kurtosis (NumPy array)
#############################
def fTG_k2erfca(kurtosis):
	erfca0 = 0.0 # Lower limit
	erfca2 = 1.0 # Upper limit
	k0 = 0.0 - kurtosis
	k2 = -1.2 - kurtosis
	xTol = 1.0E-8 # Toleance
	nIter = np.ceil(np.log2((erfca2-erfca0)/xTol)).astype(int)
	for iIter in range(nIter):
		erfca1 = 0.5 * (erfca0 + erfca2)
		k1 = fTG_erfca2k(erfca1) - kurtosis
		if (np.sign(k0) == np.sign(k1)):
			erfca0 = erfca1
			k0 = k1
		else:
			erfca2 = erfca1
			k2 = k1
	return erfca1

################
# Demonstration
################
startPoint = -1.1
endPoint = 0.0
step = 0.1
kList = np.arange(startPoint, endPoint, step)
erfcaList = np.array([])
aList = np.array([])
for i in range(kList.size):
	k = kList[i]
	erfca = fTG_k2erfca(k)
	a = sp.erfcinv(erfca)
	erfcaList = np.append(erfcaList,erfca)
	aList = np.append(aList,a)
print('k = ',kList)
print('erfc(a) = ',erfcaList)
print('a = ',aList)
